# INIsort - sorts INI files and checks for multiple entries
# 2003-01-13 Oliver Betz www.oliverbetz.de
# Based on code from Brad Baxter
# (see Usenet posting <Pine.A41.3.96.980417103111.63622N-100000@ginger.libs.uga.edu>)

# you don't need a full perl installation to run this script, since it
# doesn't need any modules. Activstate's Perl.exe and perl58.dll
# are sufficient, together <400KB compressed

# use strict; # don't use "strict" when running with minimum perl

my %inihash = ();
my( $value, $key, $section, $lastsection) = ( '' );
my $stdouterr = 1;
my $outfilerr = 0; # set as you like

if($ARGV[0] eq ''){
  print STDERR "Script to sort .INI files\n";
  print STDERR "Usage: $0 infile [outfile]\n";
  exit;
}

if((-f($ARGV[0])) == 0){
  print STDERR "couldn\'t open $ARGV[0]\n";
  exit;
}

### load the .ini file data
&ini($ARGV[0], \%inihash );

if($ARGV[1] ne ''){
  open(OUTFILE, ">$ARGV[1]") or die "couldn\'t open $ARGV[1]";
}
else{
  open(OUTFILE, ">-") or die "couldn\'t write to STDOUT";
  $stdouterr = 0; # clean output
};

### print sections in alpha order, without subroutines
foreach $section ( sort keys %inihash ) {
  print OUTFILE "\n" if($lastsection ne '');
  print OUTFILE "[$section]\n" if($section ne ''); # keys before section
  foreach $key (sort keys %{$inihash{$section}}){
    if(@{$inihash{$section}{$key}} > 1){
      print OUTFILE ";Error: duplicate key\n" if $outfilerr;
      print "Error: duplicate key [$section]->$key\n" if $stdouterr;
    }
    foreach $value (sort @{$inihash{$section}{$key}}) {
      print OUTFILE "$key = $value\n";
    }  # foreach value
  }  # foreach key
  $lastsection = $section;
}  # foreach section
close OUTFILE;

exit;

#######################################################################
### ini: read a .ini file and load into a hash of hashes of arrays
sub ini {
  my( $file, $hash_ref) = @_;
  my( $linein, $section ) = ( '');

  open( FILE, $file ) or die "Can't open $file";
  while(<FILE>) {
    next if $_ =~ /^[#;]/;  # skip comments, adjust for what you expect

    chomp;
    if( $_ =~ /^\s*\[([^\]]+)\]/ ) {
      $section = $1;
      push( @{$$hash_ref{$section}{$1}}, () );
      # "add" empty table to create hash if it didn't exist
    }  # if section name
    elsif($_ =~ /^([^=]+?)\s*=\s*(.*)\s*$/ ){
      push( @{$$hash_ref{$section}{$1}}, $2 );
    }  # elsif key
  }  # while
  close( FILE );
}  # end sub ini

__END__

